package rvl.swv;
import rvl.swv.*;
import java.io.*;

/**
  Engine for SAS code.
 */
public class SASEngine extends AbstractEngine {

    boolean imlMode; // flag for keeping track of whether IML is active

    /* These are device names.  Must be 1-1 map with okfigfmts in constructor */
    String okdevices[] = new String[] {"PDFC", "PSEPSF", "PNG", "JPEG"};

    public SASEngine() {
        engineName = "SASEngine";
        codeExt = ".sas";
        outExt = ".lst";
        okfigfmts = new int[] {FigFile.PDF, FigFile.PS, FigFile.PNG, FigFile.JPG};
        startupCode = "options nocenter nodate nonumber skip=0 ls=100 formdlim=' '\n"
            + "  formchar = '|----|+|---+=|-/\\<>*';\n"
            + "title; footnote;\n\n";
    }

// special 2-argument version of putCode
    public void putCode(String line, boolean testIML) {
        if (testIML) {
            String test = line.trim().toLowerCase();
            if (test.startsWith("proc "))
                imlMode = test.startsWith("proc iml");
            else if (test.startsWith("quit") || test.startsWith("data"))
                imlMode = false;
        }
        super.putCode(line);
    }

// Override superclass method to put iml checking true by default
    public void putCode(String line) {
        putCode(line, true);
    }

    public void putStartup(boolean weaving) {
        super.putStartup(weaving);
        if (weaving) {
            putCode("proc iml;", false);
            imlMode = true;
        }
    }

    public void putSeparator(String text) {
        if (!imlMode) {
            imlMode = true;
            putCode("proc iml;", false);
        }
        putCode("print \"" + text + "\";", false);
    }

    public void putExpr(String expr) {
        putCode("print (" + expr + ");");
    }

    public FigFile setupFig(String chunkName) {
        FigFile fig = super.setupFig(chunkName);
        if (fig != null) {
            String dev = null;
            for (int i=0; i<okfigfmts.length; i++)
                if (fig.getFormat() == okfigfmts[i]) dev = okdevices[i];
            putCode("filename grafout \"" + fig.getFileName() + "\";", false);
            putCode("goptions reset=goptions;", false);
            putCode("goptions device=" + dev + " gsfname=grafout gsfmode=replace;", false);
            putCode("goptions noborder horigin=.2in vorigin=.2in;", false);
            putCode("goptions xmax = " + fig.getWidth()
                + "in ymax = " + fig.getHeight() + "in;", false);
            putCode("goptions hsize = " + (fig.getWidth() - .2)
                + "in vsize = " + (fig.getHeight() - .2) + "in;", false);
          // SAS/GRAPH docs say I don't need this, but I do.  This essentially sets font size to 6 lines per inch
            putCode("goptions vpos = " + (int)(6*fig.getHeight()) + ";", false);
        }
        return fig;
    }

    public void closeFig() {
        if (!imlMode) putCode("run; quit;");
    }


} // ----- END OF SASEngine -------

