package rvl.swv;
import rvl.swv.*;

/**
 * Engine for Stata
 */

public class StataEngine extends AbstractEngine {

    FigFile currentFig;

    public StataEngine() {
        engineName = "StataEngine";
        codeExt = ".do";
        outExt = ".log";
        okfigfmts = new int[] {FigFile.PNG, FigFile.EPS};
    }

    public void putStartup(boolean weaving) {
        startupCode = "cd " + parent.getDir()
            + "\n\nprogram define SWVrun\n//-------\n";
        endCode = "\n//--------------\nend\n\nSWVrun\nlog using stata_check.smcl\n";
        super.putStartup(weaving);
    }

// This is a little tricky because the commands show up in the
// output file.  Need to keep those from matching the bookmarks
    public void putSeparator(String text) {
        text = text.substring(0,6) + "\" \"" + text.substring(6);
        putCode("display \"" + text + "\"");
    }

    public void putExpr(String expr) {
        putCode("display " + expr);
    }

// Figure export code all goes afterward... just need to define
// what it will look like.
    public FigFile setupFig(String chunkName) {
        currentFig = super.setupFig(chunkName);
        return currentFig;
    }

    public void closeFig() {
        if (currentFig == null) return;

        putCode("graph display, xsize(" + currentFig.getWidth()
            + ") ysize(" + currentFig.getHeight() + ")");
        String extra = "";
        if (currentFig.getFormat() != FigFile.EPS) {
            int pxw = (int)(72*currentFig.getWidth()),
                pxh = (int)(72*currentFig.getHeight());
            extra = " width(" + pxw + ") height(" + pxh + ")";
        }
        putCode("graph export " + currentFig.getFileName()
            + ", " + extra + " replace");
    }

}